# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="RedshiftConfig")


@attr.s(auto_attribs=True)
class RedshiftConfig:
    """Configuration parameters for defining a Redshift sink.

    Attributes:
        access_key_id (str):
        cluster_identifier (str):
        db_user (str):
        region (str):
        secret_access_key (str):
        db_name (Union[Unset, str]): Database name.

            Only required if not using transformations.
        schema_name (Union[Unset, None, str]): Schema name.

            Only used if not using transformations.
        table_name (Union[Unset, str]): Table name.

            Only required if not using transformations.
    """

    access_key_id: str
    cluster_identifier: str
    db_user: str
    region: str
    secret_access_key: str
    db_name: Union[Unset, str] = UNSET
    schema_name: Union[Unset, None, str] = UNSET
    table_name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        access_key_id = self.access_key_id
        cluster_identifier = self.cluster_identifier
        db_user = self.db_user
        region = self.region
        secret_access_key = self.secret_access_key
        db_name = self.db_name
        schema_name = self.schema_name
        table_name = self.table_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "accessKeyId": access_key_id,
                "clusterIdentifier": cluster_identifier,
                "dbUser": db_user,
                "region": region,
                "secretAccessKey": secret_access_key,
            }
        )
        if db_name is not UNSET:
            field_dict["dbName"] = db_name
        if schema_name is not UNSET:
            field_dict["schemaName"] = schema_name
        if table_name is not UNSET:
            field_dict["tableName"] = table_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        access_key_id = d.pop("accessKeyId")

        cluster_identifier = d.pop("clusterIdentifier")

        db_user = d.pop("dbUser")

        region = d.pop("region")

        secret_access_key = d.pop("secretAccessKey")

        db_name = d.pop("dbName", UNSET)

        schema_name = d.pop("schemaName", UNSET)

        table_name = d.pop("tableName", UNSET)

        redshift_config = cls(
            access_key_id=access_key_id,
            cluster_identifier=cluster_identifier,
            db_user=db_user,
            region=region,
            secret_access_key=secret_access_key,
            db_name=db_name,
            schema_name=schema_name,
            table_name=table_name,
        )

        redshift_config.additional_properties = d
        return redshift_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="SnowflakeConfig")


@attr.s(auto_attribs=True)
class SnowflakeConfig:
    """Configuration parameters for defining a Snowflake sink.

    Attributes:
        account_identifier (str): Snowflake account identifier, which includes both the organization and account IDs
            separated by a hyphen.
        private_key (str): PEM-encoded private key used for signing token-based requests to the Snowflake API.

            Beginning/end delimiters are not required.
        user_id (str): The Snowflake user id.
        db_name (Union[Unset, str]): Database name.

            Only required if not using transformations.
        schema_name (Union[Unset, str]): Schema name.

            Only required if not using transformations.
        table_name (Union[Unset, str]): Table name.

            Only required if not using transformations.
    """

    account_identifier: str
    private_key: str
    user_id: str
    db_name: Union[Unset, str] = UNSET
    schema_name: Union[Unset, str] = UNSET
    table_name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        account_identifier = self.account_identifier
        private_key = self.private_key
        user_id = self.user_id
        db_name = self.db_name
        schema_name = self.schema_name
        table_name = self.table_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "accountIdentifier": account_identifier,
                "privateKey": private_key,
                "userId": user_id,
            }
        )
        if db_name is not UNSET:
            field_dict["dbName"] = db_name
        if schema_name is not UNSET:
            field_dict["schemaName"] = schema_name
        if table_name is not UNSET:
            field_dict["tableName"] = table_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        account_identifier = d.pop("accountIdentifier")

        private_key = d.pop("privateKey")

        user_id = d.pop("userId")

        db_name = d.pop("dbName", UNSET)

        schema_name = d.pop("schemaName", UNSET)

        table_name = d.pop("tableName", UNSET)

        snowflake_config = cls(
            account_identifier=account_identifier,
            private_key=private_key,
            user_id=user_id,
            db_name=db_name,
            schema_name=schema_name,
            table_name=table_name,
        )

        snowflake_config.additional_properties = d
        return snowflake_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import asyncio
import random
from http import HTTPStatus
from time import sleep
from typing import Any, Dict, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_error import HttpError
from ...models.http_validation_error import HTTPValidationError
from ...models.sink_out_type_0 import SinkOutType0
from ...models.sink_out_type_1 import SinkOutType1
from ...models.sink_out_type_2 import SinkOutType2
from ...models.sink_out_type_3 import SinkOutType3
from ...types import Response


def _get_kwargs(
    app_id: str,
    sink_id: str,
    *,
    client: AuthenticatedClient,
) -> Dict[str, Any]:
    url = "{}/api/v1/app/{app_id}/sink/{sink_id}".format(client.base_url, app_id=app_id, sink_id=sink_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
    }


def _parse_response(
    *, client: Client, response: httpx.Response
) -> Union["SinkOutType0", "SinkOutType1", "SinkOutType2", "SinkOutType3"]:
    if response.status_code == HTTPStatus.OK:

        def _parse_response_200(data: object) -> Union["SinkOutType0", "SinkOutType1", "SinkOutType2", "SinkOutType3"]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_sink_out_type_0 = SinkOutType0.from_dict(data)

                return componentsschemas_sink_out_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_sink_out_type_1 = SinkOutType1.from_dict(data)

                return componentsschemas_sink_out_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_sink_out_type_2 = SinkOutType2.from_dict(data)

                return componentsschemas_sink_out_type_2
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_sink_out_type_3 = SinkOutType3.from_dict(data)

            return componentsschemas_sink_out_type_3

        response_200 = _parse_response_200(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.FORBIDDEN:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.NOT_FOUND:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.CONFLICT:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        raise HTTPValidationError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        raise HttpError.init_exception(response.json(), response.status_code)
    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Client, response: httpx.Response
) -> Response[Union["SinkOutType0", "SinkOutType1", "SinkOutType2", "SinkOutType3"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def request_sync_detailed(
    app_id: str,
    sink_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[Union["SinkOutType0", "SinkOutType1", "SinkOutType2", "SinkOutType3"]]:
    """Get Sink

     Get a sink.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        sink_id (str): The ep's ID or UID Example: unique-ep-identifier.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union['SinkOutType0', 'SinkOutType1', 'SinkOutType2', 'SinkOutType3']]
    """

    kwargs = _get_kwargs(
        app_id=app_id,
        sink_id=sink_id,
        client=client,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )
    for retry_count, sleep_time in enumerate(client.retry_schedule):
        if response.status_code < 500:
            break

        sleep(sleep_time)
        kwargs["headers"]["svix-retry-count"] = str(retry_count)
        response = httpx.request(
            verify=client.verify_ssl,
            **kwargs,
        )

    return _build_response(client=client, response=response)


def request_sync(
    app_id: str,
    sink_id: str,
    *,
    client: AuthenticatedClient,
) -> Union["SinkOutType0", "SinkOutType1", "SinkOutType2", "SinkOutType3"]:
    """Get Sink

     Get a sink.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        sink_id (str): The ep's ID or UID Example: unique-ep-identifier.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union['SinkOutType0', 'SinkOutType1', 'SinkOutType2', 'SinkOutType3']
    """

    return request_sync_detailed(
        app_id=app_id,
        sink_id=sink_id,
        client=client,
    ).parsed


async def request_asyncio_detailed(
    app_id: str,
    sink_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[Union["SinkOutType0", "SinkOutType1", "SinkOutType2", "SinkOutType3"]]:
    """Get Sink

     Get a sink.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        sink_id (str): The ep's ID or UID Example: unique-ep-identifier.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union['SinkOutType0', 'SinkOutType1', 'SinkOutType2', 'SinkOutType3']]
    """

    kwargs = _get_kwargs(
        app_id=app_id,
        sink_id=sink_id,
        client=client,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

        for retry_count, sleep_time in enumerate(client.retry_schedule):
            if response.status_code < 500:
                break

            await asyncio.sleep(sleep_time)
            kwargs["headers"]["svix-retry-count"] = str(retry_count)
            response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def request_asyncio(
    app_id: str,
    sink_id: str,
    *,
    client: AuthenticatedClient,
) -> Union["SinkOutType0", "SinkOutType1", "SinkOutType2", "SinkOutType3"]:
    """Get Sink

     Get a sink.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        sink_id (str): The ep's ID or UID Example: unique-ep-identifier.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union['SinkOutType0', 'SinkOutType1', 'SinkOutType2', 'SinkOutType3']
    """

    return (
        await request_asyncio_detailed(
            app_id=app_id,
            sink_id=sink_id,
            client=client,
        )
    ).parsed



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="SinkTransformIn")


@attr.s(auto_attribs=True)
class SinkTransformIn:
    """
    Attributes:
        code (Union[Unset, None, str]):
        enabled (Union[Unset, bool]):
    """

    code: Union[Unset, None, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        enabled = self.enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if code is not UNSET:
            field_dict["code"] = code
        if enabled is not UNSET:
            field_dict["enabled"] = enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        code = d.pop("code", UNSET)

        enabled = d.pop("enabled", UNSET)

        sink_transform_in = cls(
            code=code,
            enabled=enabled,
        )

        sink_transform_in.additional_properties = d
        return sink_transform_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TransformationTemplateKind(str, Enum):
    CUSTOM = "Custom"
    CUSTOMERIO = "CustomerIO"
    DISCORD = "Discord"
    HUBSPOT = "Hubspot"
    INNGEST = "Inngest"
    SALESFORCE = "Salesforce"
    SEGMENT = "Segment"
    SLACK = "Slack"
    TEAMS = "Teams"
    TRIGGERDEV = "TriggerDev"
    WINDMILL = "Windmill"
    ZAPIER = "Zapier"

    def __str__(self) -> str:
        return str(self.value)



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.duration import Duration


T = TypeVar("T", bound="RetryScheduleInOut")


@attr.s(auto_attribs=True)
class RetryScheduleInOut:
    """
    Attributes:
        retry_schedule (Union[Unset, List['Duration']]):
    """

    retry_schedule: Union[Unset, List["Duration"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        retry_schedule: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.retry_schedule, Unset):
            retry_schedule = []
            for componentsschemas_retry_schedule_item_data in self.retry_schedule:
                componentsschemas_retry_schedule_item = componentsschemas_retry_schedule_item_data.to_dict()

                retry_schedule.append(componentsschemas_retry_schedule_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if retry_schedule is not UNSET:
            field_dict["retrySchedule"] = retry_schedule

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.duration import Duration

        d = src_dict.copy()
        retry_schedule = []
        _retry_schedule = d.pop("retrySchedule", UNSET)
        for componentsschemas_retry_schedule_item_data in _retry_schedule or []:
            componentsschemas_retry_schedule_item = Duration.from_dict(componentsschemas_retry_schedule_item_data)

            retry_schedule.append(componentsschemas_retry_schedule_item)

        retry_schedule_in_out = cls(
            retry_schedule=retry_schedule,
        )

        retry_schedule_in_out.additional_properties = d
        return retry_schedule_in_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CustomStringsOverride")


@attr.s(auto_attribs=True)
class CustomStringsOverride:
    """
    Attributes:
        channels_help (Union[Unset, None, str]):
        channels_many (Union[Unset, None, str]):
        channels_one (Union[Unset, None, str]):
    """

    channels_help: Union[Unset, None, str] = UNSET
    channels_many: Union[Unset, None, str] = UNSET
    channels_one: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        channels_help = self.channels_help
        channels_many = self.channels_many
        channels_one = self.channels_one

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if channels_help is not UNSET:
            field_dict["channelsHelp"] = channels_help
        if channels_many is not UNSET:
            field_dict["channelsMany"] = channels_many
        if channels_one is not UNSET:
            field_dict["channelsOne"] = channels_one

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        channels_help = d.pop("channelsHelp", UNSET)

        channels_many = d.pop("channelsMany", UNSET)

        channels_one = d.pop("channelsOne", UNSET)

        custom_strings_override = cls(
            channels_help=channels_help,
            channels_many=channels_many,
            channels_one=channels_one,
        )

        custom_strings_override.additional_properties = d
        return custom_strings_override

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="AuthTokenOut")


@attr.s(auto_attribs=True)
class AuthTokenOut:
    """
    Attributes:
        created_at (datetime.datetime):
        id (str): The key's ID Example: key_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        token (str):
        expires_at (Union[Unset, None, datetime.datetime]):
        name (Union[Unset, None, str]):
        scopes (Union[Unset, None, List[str]]):
    """

    created_at: datetime.datetime
    id: str
    token: str
    expires_at: Union[Unset, None, datetime.datetime] = UNSET
    name: Union[Unset, None, str] = UNSET
    scopes: Union[Unset, None, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        created_at = self.created_at.isoformat()

        id = self.id
        token = self.token
        expires_at: Union[Unset, None, str] = UNSET
        if not isinstance(self.expires_at, Unset):
            expires_at = self.expires_at.isoformat() if self.expires_at else None

        name = self.name
        scopes: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.scopes, Unset):
            if self.scopes is None:
                scopes = None
            else:
                scopes = self.scopes

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "id": id,
                "token": token,
            }
        )
        if expires_at is not UNSET:
            field_dict["expiresAt"] = expires_at
        if name is not UNSET:
            field_dict["name"] = name
        if scopes is not UNSET:
            field_dict["scopes"] = scopes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        created_at = isoparse(d.pop("createdAt"))

        id = d.pop("id")

        token = d.pop("token")

        _expires_at = d.pop("expiresAt", UNSET)
        expires_at: Union[Unset, None, datetime.datetime]
        if _expires_at is None:
            expires_at = None
        elif isinstance(_expires_at, Unset):
            expires_at = UNSET
        else:
            expires_at = isoparse(_expires_at)

        name = d.pop("name", UNSET)

        scopes = cast(List[str], d.pop("scopes", UNSET))

        auth_token_out = cls(
            created_at=created_at,
            id=id,
            token=token,
            expires_at=expires_at,
            name=name,
            scopes=scopes,
        )

        auth_token_out.additional_properties = d
        return auth_token_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# TODO - remove this special case when we fix the generated code for empty openapi structs
import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

T = TypeVar("T", bound="MessageAttemptFailedData")


@attr.s(auto_attribs=True)
class MessageAttemptFailedData:
    """
    Attributes:
        id (str): The attempt's ID Example: atmpt_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        response_status_code (int):
        timestamp (datetime.datetime):
    """

    id: str
    response_status_code: int
    timestamp: datetime.datetime
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        response_status_code = self.response_status_code
        timestamp = self.timestamp.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "responseStatusCode": response_status_code,
                "timestamp": timestamp,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        response_status_code = d.pop("responseStatusCode")

        timestamp = isoparse(d.pop("timestamp"))

        message_attempt_failed_data = cls(
            id=id,
            response_status_code=response_status_code,
            timestamp=timestamp,
        )

        message_attempt_failed_data.additional_properties = d
        return message_attempt_failed_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import IntEnum


class StatusCodeClass(IntEnum):
    VALUE_0 = 0
    VALUE_100 = 100
    VALUE_200 = 200
    VALUE_300 = 300
    VALUE_400 = 400
    VALUE_500 = 500

    def __str__(self) -> str:
        return str(self.value)



import asyncio
import random
from http import HTTPStatus
from time import sleep
from typing import Any, Dict, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_error import HttpError
from ...models.http_validation_error import HTTPValidationError
from ...models.message_in import MessageIn
from ...models.message_out import MessageOut
from ...types import UNSET, Response, Unset


def _get_kwargs(
    app_id: str,
    *,
    client: AuthenticatedClient,
    json_body: MessageIn,
    with_content: Union[Unset, None, bool] = True,
    idempotency_key: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/app/{app_id}/msg".format(client.base_url, app_id=app_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    if not isinstance(idempotency_key, Unset):
        headers["idempotency-key"] = idempotency_key

    params: Dict[str, Any] = {}
    params["with_content"] = with_content

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> MessageOut:
    if response.status_code == HTTPStatus.ACCEPTED:
        response_202 = MessageOut.from_dict(response.json())

        return response_202
    if response.status_code == HTTPStatus.BAD_REQUEST:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.FORBIDDEN:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.NOT_FOUND:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.CONFLICT:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.REQUEST_ENTITY_TOO_LARGE:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        raise HTTPValidationError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        raise HttpError.init_exception(response.json(), response.status_code)
    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(*, client: Client, response: httpx.Response) -> Response[MessageOut]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def request_sync_detailed(
    app_id: str,
    *,
    client: AuthenticatedClient,
    json_body: MessageIn,
    with_content: Union[Unset, None, bool] = True,
    idempotency_key: Union[Unset, str] = UNSET,
) -> Response[MessageOut]:
    """Create Message

     Creates a new message and dispatches it to all of the application's endpoints.

    The `eventId` is an optional custom unique ID. It's verified to be unique only up to a day, after
    that no verification will be made.
    If a message with the same `eventId` already exists for the application, a 409 conflict error will
    be returned.

    The `eventType` indicates the type and schema of the event. All messages of a certain `eventType`
    are expected to have the same schema. Endpoints can choose to only listen to specific event types.
    Messages can also have `channels`, which similar to event types let endpoints filter by them. Unlike
    event types, messages can have multiple channels, and channels don't imply a specific message
    content or schema.

    The `payload` property is the webhook's body (the actual webhook message). Svix supports payload
    sizes of up to ~350kb, though it's generally a good idea to keep webhook payloads small, probably no
    larger than 40kb.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        with_content (Union[Unset, None, bool]): When `true` message payloads are included in the
            response Default: True.
        idempotency_key (Union[Unset, str]):
        json_body (MessageIn):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[MessageOut]
    """

    kwargs = _get_kwargs(
        app_id=app_id,
        client=client,
        json_body=json_body,
        with_content=with_content,
        idempotency_key=idempotency_key,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )
    for retry_count, sleep_time in enumerate(client.retry_schedule):
        if response.status_code < 500:
            break

        sleep(sleep_time)
        kwargs["headers"]["svix-retry-count"] = str(retry_count)
        response = httpx.request(
            verify=client.verify_ssl,
            **kwargs,
        )

    return _build_response(client=client, response=response)


def request_sync(
    app_id: str,
    *,
    client: AuthenticatedClient,
    json_body: MessageIn,
    with_content: Union[Unset, None, bool] = True,
    idempotency_key: Union[Unset, str] = UNSET,
) -> MessageOut:
    """Create Message

     Creates a new message and dispatches it to all of the application's endpoints.

    The `eventId` is an optional custom unique ID. It's verified to be unique only up to a day, after
    that no verification will be made.
    If a message with the same `eventId` already exists for the application, a 409 conflict error will
    be returned.

    The `eventType` indicates the type and schema of the event. All messages of a certain `eventType`
    are expected to have the same schema. Endpoints can choose to only listen to specific event types.
    Messages can also have `channels`, which similar to event types let endpoints filter by them. Unlike
    event types, messages can have multiple channels, and channels don't imply a specific message
    content or schema.

    The `payload` property is the webhook's body (the actual webhook message). Svix supports payload
    sizes of up to ~350kb, though it's generally a good idea to keep webhook payloads small, probably no
    larger than 40kb.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        with_content (Union[Unset, None, bool]): When `true` message payloads are included in the
            response Default: True.
        idempotency_key (Union[Unset, str]):
        json_body (MessageIn):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        MessageOut
    """

    return request_sync_detailed(
        app_id=app_id,
        client=client,
        json_body=json_body,
        with_content=with_content,
        idempotency_key=idempotency_key,
    ).parsed


async def request_asyncio_detailed(
    app_id: str,
    *,
    client: AuthenticatedClient,
    json_body: MessageIn,
    with_content: Union[Unset, None, bool] = True,
    idempotency_key: Union[Unset, str] = UNSET,
) -> Response[MessageOut]:
    """Create Message

     Creates a new message and dispatches it to all of the application's endpoints.

    The `eventId` is an optional custom unique ID. It's verified to be unique only up to a day, after
    that no verification will be made.
    If a message with the same `eventId` already exists for the application, a 409 conflict error will
    be returned.

    The `eventType` indicates the type and schema of the event. All messages of a certain `eventType`
    are expected to have the same schema. Endpoints can choose to only listen to specific event types.
    Messages can also have `channels`, which similar to event types let endpoints filter by them. Unlike
    event types, messages can have multiple channels, and channels don't imply a specific message
    content or schema.

    The `payload` property is the webhook's body (the actual webhook message). Svix supports payload
    sizes of up to ~350kb, though it's generally a good idea to keep webhook payloads small, probably no
    larger than 40kb.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        with_content (Union[Unset, None, bool]): When `true` message payloads are included in the
            response Default: True.
        idempotency_key (Union[Unset, str]):
        json_body (MessageIn):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[MessageOut]
    """

    kwargs = _get_kwargs(
        app_id=app_id,
        client=client,
        json_body=json_body,
        with_content=with_content,
        idempotency_key=idempotency_key,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

        for retry_count, sleep_time in enumerate(client.retry_schedule):
            if response.status_code < 500:
                break

            await asyncio.sleep(sleep_time)
            kwargs["headers"]["svix-retry-count"] = str(retry_count)
            response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def request_asyncio(
    app_id: str,
    *,
    client: AuthenticatedClient,
    json_body: MessageIn,
    with_content: Union[Unset, None, bool] = True,
    idempotency_key: Union[Unset, str] = UNSET,
) -> MessageOut:
    """Create Message

     Creates a new message and dispatches it to all of the application's endpoints.

    The `eventId` is an optional custom unique ID. It's verified to be unique only up to a day, after
    that no verification will be made.
    If a message with the same `eventId` already exists for the application, a 409 conflict error will
    be returned.

    The `eventType` indicates the type and schema of the event. All messages of a certain `eventType`
    are expected to have the same schema. Endpoints can choose to only listen to specific event types.
    Messages can also have `channels`, which similar to event types let endpoints filter by them. Unlike
    event types, messages can have multiple channels, and channels don't imply a specific message
    content or schema.

    The `payload` property is the webhook's body (the actual webhook message). Svix supports payload
    sizes of up to ~350kb, though it's generally a good idea to keep webhook payloads small, probably no
    larger than 40kb.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        with_content (Union[Unset, None, bool]): When `true` message payloads are included in the
            response Default: True.
        idempotency_key (Union[Unset, str]):
        json_body (MessageIn):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        MessageOut
    """

    return (
        await request_asyncio_detailed(
            app_id=app_id,
            client=client,
            json_body=json_body,
            with_content=with_content,
            idempotency_key=idempotency_key,
        )
    ).parsed



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.border_radius_enum import BorderRadiusEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="BorderRadiusConfig")


@attr.s(auto_attribs=True)
class BorderRadiusConfig:
    """
    Attributes:
        button (Union[Unset, BorderRadiusEnum]):
        card (Union[Unset, BorderRadiusEnum]):
        input_ (Union[Unset, BorderRadiusEnum]):
    """

    button: Union[Unset, BorderRadiusEnum] = UNSET
    card: Union[Unset, BorderRadiusEnum] = UNSET
    input_: Union[Unset, BorderRadiusEnum] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        button: Union[Unset, str] = UNSET
        if not isinstance(self.button, Unset):
            button = self.button.value

        card: Union[Unset, str] = UNSET
        if not isinstance(self.card, Unset):
            card = self.card.value

        input_: Union[Unset, str] = UNSET
        if not isinstance(self.input_, Unset):
            input_ = self.input_.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if button is not UNSET:
            field_dict["button"] = button
        if card is not UNSET:
            field_dict["card"] = card
        if input_ is not UNSET:
            field_dict["input"] = input_

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _button = d.pop("button", UNSET)
        button: Union[Unset, BorderRadiusEnum]
        if isinstance(_button, Unset):
            button = UNSET
        else:
            button = BorderRadiusEnum(_button)

        _card = d.pop("card", UNSET)
        card: Union[Unset, BorderRadiusEnum]
        if isinstance(_card, Unset):
            card = UNSET
        else:
            card = BorderRadiusEnum(_card)

        _input_ = d.pop("input", UNSET)
        input_: Union[Unset, BorderRadiusEnum]
        if isinstance(_input_, Unset):
            input_ = UNSET
        else:
            input_ = BorderRadiusEnum(_input_)

        border_radius_config = cls(
            button=button,
            card=card,
            input_=input_,
        )

        border_radius_config.additional_properties = d
        return border_radius_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
Svix

The Svix server API documentation  # noqa: E501

The version of the OpenAPI document: 0.8.1
Generated by: https://openapi-generator.tech
"""

import os
import re
import codecs
from setuptools import setup, find_packages  # noqa: H301

# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools


def read_file(filepath):
    """Read content from a UTF-8 encoded text file."""
    with codecs.open(filepath, "rb", "utf-8") as file_handle:
        return file_handle.read()


PKG_NAME = "svix"
PKG_DIR = os.path.abspath(os.path.dirname(__file__))
META_PATH = os.path.join(PKG_DIR, PKG_NAME, "__init__.py")
META_CONTENTS = read_file(META_PATH)
PKG_REQUIRES = [
    "httpx >=0.23.0",
    "attrs >=21.3.0",
    "python-dateutil",
    "Deprecated",
    "types-python-dateutil",
    "types-Deprecated",
]


def find_meta(meta):
    """Extract __*meta*__ from META_CONTENTS."""
    meta_match = re.search(
        r"^__{meta}__\s+=\s+['\"]([^'\"]*)['\"]".format(meta=meta), META_CONTENTS, re.M
    )

    if meta_match:
        return meta_match.group(1)
    raise RuntimeError(f"Unable to find __{meta}__ string in package meta file")


def is_canonical_version(version):
    """Check if a version string is in the canonical format of PEP 440."""
    pattern = (
        r"^([1-9][0-9]*!)?(0|[1-9][0-9]*)(\.(0|[1-9][0-9]*))"
        r"*((a|b|rc)(0|[1-9][0-9]*))?(\.post(0|[1-9][0-9]*))"
        r"?(\.dev(0|[1-9][0-9]*))?$"
    )
    return re.match(pattern, version) is not None


def get_version_string():
    """Return package version as listed in `__version__` in meta file."""
    # Parse version string
    version_string = find_meta("version")

    # Check validity
    if not is_canonical_version(version_string):
        message = (
            'The detected version string "{}" is not in canonical '
            "format as defined in PEP 440.".format(version_string)
        )
        raise ValueError(message)

    return version_string


PKG_README = read_file(os.path.join(os.path.dirname(__file__), "README.md"))

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name=PKG_NAME,
    version=get_version_string(),
    description="Svix webhooks API client and webhook verification library",
    author="Svix",
    author_email="development@svix.com",
    url="https://www.svix.com",
    license="MIT",
    keywords=[
        "svix",
        "diahook",
        "webhooks",
    ],
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development",
        "Typing :: Typed",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    python_requires=">=3.6",
    install_requires=PKG_REQUIRES,
    zip_safe=False,
    packages=find_packages(exclude=["test", "tests"]),
    package_data={
        "": ["py.typed"],
    },
    long_description=PKG_README,
    long_description_content_type="text/markdown",
)



# TODO - remove this special case when we fix the generated code for empty openapi structs
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AttemptStatisticsData")


@attr.s(auto_attribs=True)
class AttemptStatisticsData:
    """
    Attributes:
        failure_count (Union[Unset, None, List[int]]):
        success_count (Union[Unset, None, List[int]]):
    """

    failure_count: Union[Unset, None, List[int]] = UNSET
    success_count: Union[Unset, None, List[int]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        failure_count: Union[Unset, None, List[int]] = UNSET
        if not isinstance(self.failure_count, Unset):
            if self.failure_count is None:
                failure_count = None
            else:
                failure_count = self.failure_count

        success_count: Union[Unset, None, List[int]] = UNSET
        if not isinstance(self.success_count, Unset):
            if self.success_count is None:
                success_count = None
            else:
                success_count = self.success_count

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if failure_count is not UNSET:
            field_dict["failureCount"] = failure_count
        if success_count is not UNSET:
            field_dict["successCount"] = success_count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        failure_count = cast(List[int], d.pop("failureCount", UNSET))

        success_count = cast(List[int], d.pop("successCount", UNSET))

        attempt_statistics_data = cls(
            failure_count=failure_count,
            success_count=success_count,
        )

        attempt_statistics_data.additional_properties = d
        return attempt_statistics_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import asyncio
import random
from http import HTTPStatus
from time import sleep
from typing import Any, Dict, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_error import HttpError
from ...models.http_validation_error import HTTPValidationError
from ...types import Response


def _get_kwargs(
    app_id: str,
    *,
    client: AuthenticatedClient,
) -> Dict[str, Any]:
    url = "{}/api/v1/app/{app_id}".format(client.base_url, app_id=app_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "delete",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Any:
    if response.status_code == HTTPStatus.NO_CONTENT:
        response_204 = cast(Any, None)
        return response_204
    if response.status_code == HTTPStatus.BAD_REQUEST:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.FORBIDDEN:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.NOT_FOUND:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.CONFLICT:
        raise HttpError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        raise HTTPValidationError.init_exception(response.json(), response.status_code)
    if response.status_code == HTTPStatus.TOO_MANY_REQUESTS:
        raise HttpError.init_exception(response.json(), response.status_code)
    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(*, client: Client, response: httpx.Response) -> Response[None]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def request_sync_detailed(
    app_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[None]:
    """Delete Application

     Delete an application.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[None]
    """

    kwargs = _get_kwargs(
        app_id=app_id,
        client=client,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )
    for retry_count, sleep_time in enumerate(client.retry_schedule):
        if response.status_code < 500:
            break

        sleep(sleep_time)
        kwargs["headers"]["svix-retry-count"] = str(retry_count)
        response = httpx.request(
            verify=client.verify_ssl,
            **kwargs,
        )

    return _build_response(client=client, response=response)


def request_sync(
    app_id: str,
    *,
    client: AuthenticatedClient,
) -> None:
    """Delete Application

     Delete an application.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        None
    """

    return request_sync_detailed(
        app_id=app_id,
        client=client,
    ).parsed


async def request_asyncio_detailed(
    app_id: str,
    *,
    client: AuthenticatedClient,
) -> Response[None]:
    """Delete Application

     Delete an application.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[None]
    """

    kwargs = _get_kwargs(
        app_id=app_id,
        client=client,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

        for retry_count, sleep_time in enumerate(client.retry_schedule):
            if response.status_code < 500:
                break

            await asyncio.sleep(sleep_time)
            kwargs["headers"]["svix-retry-count"] = str(retry_count)
            response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def request_asyncio(
    app_id: str,
    *,
    client: AuthenticatedClient,
) -> None:
    """Delete Application

     Delete an application.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        None
    """

    return (
        await request_asyncio_detailed(
            app_id=app_id,
            client=client,
        )
    ).parsed



# TODO - remove this special case when we fix the generated code for empty openapi structs
import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="ReplayIn")


@attr.s(auto_attribs=True)
class ReplayIn:
    """
    Attributes:
        since (datetime.datetime):
        until (Union[Unset, None, datetime.datetime]):
    """

    since: datetime.datetime
    until: Union[Unset, None, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        since = self.since.isoformat()

        until: Union[Unset, None, str] = UNSET
        if not isinstance(self.until, Unset):
            until = self.until.isoformat() if self.until else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "since": since,
            }
        )
        if until is not UNSET:
            field_dict["until"] = until

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        since = isoparse(d.pop("since"))

        _until = d.pop("until", UNSET)
        until: Union[Unset, None, datetime.datetime]
        if _until is None:
            until = None
        elif isinstance(_until, Unset):
            until = UNSET
        else:
            until = isoparse(_until)

        replay_in = cls(
            since=since,
            until=until,
        )

        replay_in.additional_properties = d
        return replay_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
